<?php
/**
 * @file
 * Hooks for Cforge distribution profile.
 *
 * @todo ensure all lists show only active users with role RID_TRADER
 */

use Drupal\node\Entity\Node;
use Drupal\user\UserInterface;
use Drupal\mcapi\Entity\Wallet;
use Drupal\Core\Extension\Extension;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\Core\Render\Element;
use Drupal\Core\Url;
use Drupal\Core\Access\AccessResult;
use Drupal\user\Entity\User;
use Drupal\mcapi\Mcapi;

// Roles.
const RID_TRADER = 'trader';
const RID_COMMITTEE = 'committee';
const CFORGE_FLAVOURS = [
  'hamlets' => 'Hamlets',
  'ces' => 'CES',
  'localexchange' => 'Local Exchange',
];

/**
 * Implements hook_form_FORM_ID_alter().
 */
function cforge_form_user_login_form_alter(&$form, $form_state) {
  $request = \Drupal::request();
  if (!$request->request->has('destination')) {
    $path = \Drupal::config('cforge.settings')->get('member_frontpage');
    $form_state->setRedirect(Url::fromUri('internal:' . $path));
    $request->query->set('destination', $path);
  }
}

/**
 * Implements hook_block_view_BASE_BLOCK_ID_alter().
 *
 * Change the title of the user block.
 */
function cforge_block_view_system_menu_block_alter(&$build, $block) {
  if ($block->getPluginId() == 'system_menu_block:account') {
    $build['#configuration']['label'] = \Drupal::currentUser()->getDisplayName();
  }
}

/**
 * Implements hook_cron().
 */
function cforge_cron() {
  // Delete old sessions to keep the sessions table small.
  db_delete('sessions')
    ->condition('uid', 0)
    ->condition('timestamp', strtotime('-1 day'), '<')
    ->execute();
  db_delete('sessions')
    ->condition('timestamp', strtotime('-1 month'), '<')
    ->execute();
  // Keep the watchdog table small too.
  if (\Drupal::moduleHandler()->moduleExists('watchdog')) {
    db_delete('watchdog')
      ->condition('timestamp', strtotime('-1 week'), '<')
      ->execute();
  }
}

/**
 * Implements hook_element_info_alter().
 *
 * This is all the code from the disablepwstrength module.
 */
function cforge_element_info_alter(&$types) {
  $processes = &$types['password_confirm']['#process'];
  if (isset($processes)) {
    $position = array_search('user_form_process_password_confirm', $processes);
    if ($position !== FALSE) {
      unset($processes[$position]);
    }
  }
}

/**
 * Implements hook_entity_info_alter().
 *
 * Add new Form handlers for the user profile edit forms.
 */
function cforge_entity_type_alter(array &$entity_types) {
  if (isset($entity_types['user'])) {
    $entity_types['user']->setFormClass('profile', "Drupal\user\ProfileForm");
    $entity_types['user']->setFormClass('admin', "Drupal\user\ProfileForm");
    $entity_types['user']->setAccessClass('Drupal\cforge\NeighbourhoodUserAccess');
  }
  if (isset($entity_types['smallad'])) {
    $entity_types['smallad']->setAccessClass('Drupal\cforge\NeighbourhoodSmalladAccess');
  }
  if (isset($entity_types['smallad'])) {
    $entity_types['mcapi_wallet']->setAccessClass('Drupal\cforge\NeighbourhoodWalletAccess');
  }
  // @todo apart from these three, should super neighbours be able to use wallets?

}

/**
 * Implements hook_user_presave().
 */
function cforge_user_presave($account) {
  // If the user is being created, add the trader role.
  if ($account->isNew()) {
    $account->addRole(RID_TRADER);
  }
}

/**
 * Implements hook_form_FORM_ID_alter().
 */
function cforge_form_user_form_alter(&$form, $form_state) {
  if (isset($form['account'])) {
    $form['account']['status']['#options'][0] = t('User has not been approved or is no longer a member');
    $form['account']['status']['#options'][1] = t('User can log in normally');
    $form['account']['roles']['#options'][RID_COMMITTEE] .= ' - ' . (string) t('Can manage members, articles, pages images, & documents.');
    $form['account']['roles']['#options'][RID_TRADER] .= ' - ' . t('Can use the main currency.');
  }
  // Remove the mutiple field theme from the phone numbers.
  $form['phones']['widget'][0]['_weight']['#access'] = FALSE;
  $form['phones']['widget'][1]['_weight']['#access'] = FALSE;
  unset($form['phones']['widget']['#theme']);

  if (isset($form['picture'])) {
    $helpful = t('No spaces in filenames!');
    $form['picture']['picture_upload']['#title'] .= ' (' . $helpful . ')';
  }

}

/**
 * Implements hook field_widget_WIDGET_NAME_form_alter().
 *
 * Tailor the display of the phone multiple cardinality field.
 */
function cforge_field_widget_telephone_default_form_alter(&$element, $form_state, $context) {
  static $i = 0;
  $element['value']['#field_prefix']  = ($i % 2 == 1) ? t('Other phone:') : t('Mobile phone:');
  $i++;
}

/**
 * Implements hook field_widget_WIDGET_NAME_form_alter().
 *
 * Reformat the address widget assuming there is no choice of country and
 * neighbourhood is a dropdown. Restrict the available countries to those in
 * config.
 */
function cforge_field_widget_address_default_form_alter(&$element, $form_state, $context) {
  $element['country_code']['#type'] = 'hidden';
  unset($element['country_code']['#options']);
  $element['dependent_locality']['#options'] = cforge_list_neighbourhoods();
  $element['dependent_locality']['#type'] = 'select';
  unset($element['dependent_locality']['#size']);
}

/**
 * Implements hook_form_FORM_ID_alter().
 *
 * Tailor the account settings form.
 */
function cforge_form_user_admin_settings_alter(&$form) {
  $reg_fieldset = &$form['registration_cancellation'];
  unset(
    $reg_fieldset['#description'],
    $reg_fieldset['user_register']['#options'][USER_REGISTER_VISITORS]
  );
  $reg_fieldset['#weight'] = -5;
  $reg_fieldset['user_register']['#title'] = t('How to add new members');
  $reg_fieldset['user_register']['#options'][USER_REGISTER_ADMINISTRATORS_ONLY] = t('Only committee can create accounts');
  $reg_fieldset['user_register']['#options'][USER_REGISTER_VISITORS_ADMINISTRATIVE_APPROVAL] = t('Anyone can create an account, then a committee member must enable it');
  $reg_fieldset['user_cancel_method']['#access'] = FALSE;
  $reg_fieldset['user_email_verification']['#access'] = FALSE;

  $form['contact']['#access'] = FALSE;
  $form['admin_role']['#access'] = FALSE;

}

/**
 * Implements hook_form_FORM_ID_alter().
 *
 * Other node types have no menus at all.
 */
function cforge_form_node_page_form_alter(&$form, &$form_state) {
  $home = \Drupal::config('system.site')->get('page.front');
  // Prevent changing the alias of the home page, and remove the delete button.
  if ($form['path']['widget'][0]['alias']['#default_value'] == $home) {
    $form['path']['warning'] = [
      '#markup' => t(
        "Warning: '@name' is your site's front page.",
        ['@name' => $home]
      ),
    ];
  }
}

/**
 * Implements hook_form_node_form_alter().
 *
 * Make the descriptions of the aliases more friendly.
 */
function cforge_form_node_form_alter(&$form, $form_state) {
  $node = $form_state->getFormObject()->getEntity();
  if ($node->bundle() != 'neighbourhood') {
    $form['public'] = [
      '#title' => t('Visible to the public'),
      '#type' => 'checkbox',
      '#default_value' => cforge_node_get_anonaccess($node->id()),
    ];
    foreach (Element::children($form['actions']) as $name) {
      $actions[$name]['#submit'][] = 'cforge_node_submit';
    }
  }
  // Rename the url alias field and explain it better.
  $form['path']['widget'][0]['alias']['#title'] = t('Friendly address');
  $form['path']['widget'][0]['alias']['#description'] = t(
    "Optionally give your page a more meaningful address. E.g. 'articles-of-association'"
  );

}

/**
 * Implements hook_menu_links_discovered_alter().
 *
 * Remove unwanted menu items put in the main menu by core modules.
 */
function cforge_menu_links_discovered_alter(&$links) {
  unset(
    // Poor default in the node module.
    $links['node.add_page'],
    // Was disabled anyway.
    $links['filter.tips_all'],
    // Was disabled anyway.
    $links['search.view'],
    // @see Drupal\cforge\Routing\RouteSubscriber
    // $links['masquerade.unmasquerade'],//access bug in masquerade module beta
    // created by admin_toolbar_tools module
    $links['entity.menu.add_form']
  );
  $links['contact.site_page']['enabled'] = TRUE;
  // Having removed node.add_page its is helpful to rename the links under it.
  foreach (\Drupal::entityTypeManager()->getStorage('node_type')->loadMultiple() as $type) {
    $type_id = $type->id();
    if (isset($links['node.add.' . $type_id])) {
      $links['node.add.' . $type_id]['title'] = t(
        'Add @type',
        ['@type' => $links['node.add.' . $type_id]['title']]
      );
    }
  }
  // This works even before taxonomy is installed.
  foreach (\Drupal::configFactory()->listAll('taxonomy.vocabulary') as $id) {
    $vocab = Vocabulary::load(substr($id, 20));
    $vid = $vocab->id();
    $links['entity.taxonomy_vocabulary.overview_form.cforge.' . $vid] = [
      'title' => t('Manage @name', ['@name' => $vid]),
      'description' => $vocab->getDescription(),
      'route_name' => 'entity.taxonomy_vocabulary.overview_form',
      'route_parameters' => ['taxonomy_vocabulary' => $vid],
      'menu_name' => 'tools',
    ];
    // The admin_toolbar_tools module puts its own links...
    $key = 'entity.taxonomy_term.add_form.' . $vid;
    if (isset($links[$key])) {
      unset($links[$key]);
    }
  }
  if (isset($links['user.logout'])) {
    // This overrides admin_toolbar_tools_menu_links_discovered_alter()
    unset($links['user.logout']['parent']);
    $links['user.logout']['menu_name'] = 'account';
  }
}

/**
 * Implements hook_menu_local_actions_alter().
 *
 * @note the the menu links, actions and task that refer to it are also removed.
 */
function cforge_menu_local_actions_alter(&$items) {
  // @see Drupal\cforge\Routing\RouteSubscriber
  unset($items['entity.menu.add_form']);
}

/**
 * Implements hook_mail().
 */
function cforge_mail($key, &$message, $params) {
  if ($key == 'all') {
    $recipient = User::loadByProperties(['mail' => $message['to']]);
    // Tokens allowed were.
    $message['subject'] = $params['subject'];
    $message['body'][] = \Drupal::token()->replace($params['body'], ['user' => $recipient]);
  }
}

/**
 * Implements hook_mail_alter().
 *
 * Add the mail footer.
 */
function cforge_mail_alter(&$message) {
  if (\Drupal::currentUser()->id()) {
    if (!is_array($message['body'])) {
      drupal_set_message('problem constructing body of mail '.$message['key'], 'warning');
    }
    $message['body'][] = \Drupal::token()->replace(
      \Drupal::config('cforge.settings')->get('mail_footer'),
      $message['params'],
      [
        'language' => $message['langcode'],
        'sanitize' => FALSE,
      ]
    );
  }
}

/**
 * Implements hook_form_system_site_information_settings_alter().
 */
function cforge_form_system_site_information_settings_alter(&$form, $form_state) {
  $form['front_page']['member_frontpage'] = [
    '#title' => t("Member's front page"),
    '#description' => t("Users will be redirected to this page when they log in."),
    '#default_value' => \Drupal::config('cforge.settings')->get('member_frontpage'),
    '#element_validate' => ['cforge_member_frontpage_validate'],
  ] + $form['front_page']['site_frontpage'];
  $form['actions']['submit']['#submit'][] = 'cforge_submit_system_site_information_settings';
}

/**
 * Element validation callback.
 */
function cforge_member_frontpage_validate(&$element, $form_state) {
  if ($element['#value'] && $element['#value'][0] !== '/') {
    $form_state->setError($element, t("The path '%path' has to start with a slash.", ['%path' => $element['#value']]));
  }
}

/**
 * Form submission callback.
 */
function cforge_submit_system_site_information_settings($form, $form_state) {
  \Drupal::configFactory()->getEditable('cforge.settings')
    ->set('member_frontpage', $form_state->getValue('member_frontpage'))
    ->save();
}

/**
 * Implements hook_locale_translation_projects().
 *
 * @temp until the communityforge.net directory exists
 *
 * @see locale_translation_check_projects_local()
 */
function cforge_locale_translation_projects_alter(&$projects) {
  foreach (['cforge', 'smallads', 'mcapi'] as $module) {
    $projects[$module]['server_pattern'] = 'localhost/translations/8/%module/%project-%version.%language.po';
  }
}

/**
 * Menu callback.
 *
 * Shortcut to the backup & restore module download backup.
 *
 * @todo test when backup_migrate is available
 */
function backup_direct() {
  // @todo flush all caches
  $form_state['values'] = [
    'profile_id' => NULL,
    'destination_id' => 'download',
    'source_id' => NULL,
  ];
  drupal_form_submit('backup_migrate_ui_manual_quick_backup_form', $form_state);
  drupal_goto(BACKUP_MIGRATE_MENU_PATH . '/restore');
}

/**
 * Implements hook_form_system_modules_alter().
 *
 * Disable the cforge_hosted checkbox, if present.
 * Add Readme links to the modules page, where readme files exist.
 * Prevent the multiple exchanges module being installed.
 */
function cforge_form_system_modules_alter(&$form, &$form_state) {
  // Protect the cforge hosted module (only applies on cforge server)
  $other = (string) t('Other');
  if (\Drupal::currentUser()->id() > 1) {
    if (array_key_exists('cforge_hosted', $form['modules'][$other])) {
      $form['modules'][$other]['cforge_hosted']['enable']['#disabled'] = TRUE;
      $form['modules'][$other]['cforge_hosted']['description']['#markup'] .= strtoupper(t('Only user 1 can disable this module'));
    }
  }
  unset($form['modules'][$other]['cforge_import']);
  // Link to the readme file. (cool idea taken from the advanced help module).
  foreach (Element::children($form['modules']) as $group) {
    foreach (Element::children($form['modules'][$group]) as $module) {
      $path = DRUPAL_ROOT . '/' . drupal_get_path('module', $module);
      if (file_exists($path . '/README.txt')) {
        $form['modules'][$group][$module]['links']['help'] = [
          '#type' => 'link',
          '#title' => t('README.txt'),
          '#url' => Url::fromRoute('cforge.readme', ['module' => $module]),
          '#attributes' => [
            'class' => ['module-link'],
          ],
        ];
      }
    }
  }
  // This module is incompatible with Hamlets.
  unset($form['modules'][(string) t('Community Accounting')]['mcapi_exchanges']);
  // Impossible to change flavour so don't even show the flavours as modules.
  foreach (CFORGE_FLAVOURS as $mod => $name) {
    unset($form['modules'][(string) t('Other')][$mod]);
  }
}

/**
 * Implements hook_system_info_alter().
 *
 * Move things around on the modules page.
 */
function cforge_system_info_alter(&$info, Extension $extension, $type) {
  // Make some contrib modules into cforge features!
  switch ($extension->getName()) {
    case 'community_tasks':
      $info['package'] = (string) t('Cforge features');
  }
}

/**
 * Implements hook_form_alter().
 *
 * Clears the css cache after a modification.
 *
 * @todo test when css_injector is available
 */
function cforge_form_css_injector_edit_alter(&$form, &$form_state) {
  $form['buttons']['save']['#submit'][] = 'drupal_clear_css_cache';
  $form['buttons']['save_and_continue']['#submit'][] = 'drupal_clear_css_cache';
}

/**
 * Utility function to make batch mail a little easier for cforge dependencies.
 *
 * @param string $module
 *   The name of the modue which is responsible for the mail.
 * @param string $key
 *   The key of that mail.
 * @param array $uids
 *   The Uids of the recipients.
 * @param array $params
 *   The mail params.
 * @param \Drupal\Core\Url|string $redirect
 *   Where to redirect after tha batch is run.
 * @param string $finished
 *   The path to go to at the end of the batch.
 *
 * @todo test batch mail in d8.
 */
function cforge_batch_mail($module, $key, array $uids, $params, $redirect, $finished = '') {
  if (empty($uids)) {
    return;
  }
  $batch = [
    'title' => t('Sending @count mails', ['@count' => count($uids)]),
    // @see https://api.drupal.org/api/drupal/core!includes!form.inc/function/batch_process/8
    'batch_redirect' => $redirect,
  ];
  if ($finished) {
    $batch['finished'] = $finished;
  }
  foreach ($uids as $uid) {
    $batch['operations'][] = [
      'cforge_batch_mail_send',
      [$module, $key, $uid, $params],
    ];
  }
  batch_set($batch);
}

/**
 * Batch callback.
 */
function cforge_batch_mail_send($module, $key, $uid, $params, $progress) {
  $account = user_load($uid);
  $params['user'] = $account;
  \Drupal::service('plugin.manager.mail')->mail(
    $module,
    $key,
    $account->getEmail(),
    $account->getPreferredLangcode(),
    $params
  );
}

/**
 * Implements hook_node_access().
 */
function cforge_node_access($node, $op, $account) {
  $result = AccessResult::neutral();
  // Prevent deletion or changing name of used neighbourhoods.
  if ($node->bundle() == 'neighbourhood') {
    if ($op == 'update' || $op == 'delete') {
      // Find out if any instances of the address field have this node as their
      // neighbourhood.
      $used = \Drupal::entityQuery('user')
        ->condition('address.dependent_locality', $form_state->getFormObject()->getEntity()->label())
        ->count()
        ->execute();
      if ($used) {
        $result = AccessResult::forbidden();
      }
    }
  }
  elseif ($op == 'view') {
    $public = cforge_node_get_anonaccess($node->id());
    if ($node->bundle() == 'page') {
      $result = ($public == $account->isAnonymous()) ?
        AccessResult::allowed()->cachePerPermissions() :
        AccessResult::forbidden();
    }
    else {
      $result = $public || $account->isAuthenticated() ?
        AccessResult::allowed()->cachePerPermissions() :
        AccessResult::forbidden();
    }
  }
  return $result;
}

/**
 * Implements hook_entity_extra_field_info().
 *
 * Puts a details field used by the broadcast module and the latest module.
 */
function cforge_entity_extra_field_info() {
  $fields = [];
  $fields['user']['user']['form']['privacy'] = [
    'label' => t('Per-field privacy settings'),
    'weight' => 8,
    'visible' => TRUE,
  ];
  $fields['smallad']['offer']['display']['transact'] = [
    'label' => 'Link to credit form',
    'weight' => 10,
    'visible' => TRUE,
  ];
  $fields['smallad']['want']['display']['transact'] = [
    'label' => 'Link to billing form',
    'weight' => 10,
    'visible' => TRUE,
  ];
  return $fields;
}

/**
 * Implements hook_ENTITY_TYPE_view_alter().
 *
 * Adds a link to the smallad to jump to a prefilled payment form. This link,
 * since it uses a query, doesn't have access control as part of the route so
 * this function only shows the link when needed.
 */
function cforge_smallad_view_alter(array &$build, $entity, $display) {
  if ($display->getComponent('transact') and $entity->getOwnerId() != \Drupal::currentUser()->id()) {
    if ($wids = Mcapi::walletsOf($entity->getOwner())) {
      $form_displays = mcapi_form_displays_load();
      $query = [
        'description' => $entity->title->value,
      ];
      if ($entity->hasField(SMALLAD_CATEGORIES)) {
        $query['category'] = $entity->{SMALLAD_CATEGORIES}->getValue();
      }
      foreach ($wids as $wid) {
        if ($entity->bundle() == 'offer') {
          $direction = MCAPI_FORMS_DIR_INCOMING;
          $query['payee'] = $wid;
        }
        elseif ($entity->bundle() == 'want') {
          $direction = MCAPI_FORMS_DIR_OUTGOING;
          $query['payer'] = $wid;
        }
        $wallet = Wallet::load($wid);
        foreach ($form_displays as $form_id => $form_display) {
          $settings = $form_display->getThirdPartySettings('mcapi_forms');
          // This doesn't allow for MCAPI_FORMS_DIR_BOTH.
          if ($settings['direction'] == $direction) {
            if ($walletLinkTitle = $settings['wallet_link_title']) {
              $build['pay_links'][]['transact_' . $form_id] = [
                '#type' => 'link',
                '#title' => wallet_link_title($wallet, $walletLinkTitle),
                '#url' => Url::fromUserInput($settings['path'], ['query' => $query]),
                '#attributes' => ['style' => 'display:block;'],
              ];
            }
          }
        }
      }
    }
  }
}

/**
 * Theme preprocessor for smallads.
 *
 * Add the pay link(s).
 */
function cforge_preprocess_smallad(&$vars) {
  if (isset($vars['elements']['pay_links'])) {
    $vars['pay_links'] = render($vars['elements']['pay_links']);
  }
}

/**
 * Implements hook_ENTITY_TYPE_view_alter().
 *
 * Render the convert_link extra field on smalladswalletLinkTitle.
 *
 * @todo cache result of this
 */
function cforge_smallad_view(&$build, $smallad, $display, $view_mode) {
  if ($content = $display->getComponent('convert_link')) {
    // This only works for offers and for wants, the two default smallad types.
    // and it ignores any price field that may have been added.
    $type = $smallad->type->target_id;
    $my_wallet_ids = Mcapi::walletsOf(User::load(\Drupal::currentUser()->id()));
    $owner_wallet_ids = Mcapi::walletsOf($smallad->getOwner());
    $query = [
      'payer' => reset($my_wallet_ids),
      'payee' => reset($owner_wallet_ids),
      'title' => $smallad->title->value,
      // If we really wanted we could add a worth field here.
    ];
    if ($type == 'offer') {
      $build['convert_link'] = [
        '#type' => 'link',
        '#title' => t('Pay for this'),
        '#url' => Url::fromRoute('mcapi.1stparty.credit', [], ['query' => $query]),
      // '#cachetags' => //per session.
      ];
    }
    elseif ($type == 'want') {
      $build['convert_link'] = [
        '#type' => 'link',
        '#title' => t('Pay for this'),
        '#url' => Url::fromRoute('mcapi.1stparty.bill', [], ['query' => $query]),
      // '#cachetags' => //per session.
      ];
    }
  }
}

/**
 * Implements hook_form_taxonomy_term_categories_form_alter().
 *
 * Force the term description field to be plain text. There doesn't seem to be a
 * way of doing this by altering the baseFieldDefinition.
 */
function cforge_form_taxonomy_term_categories_form_alter(&$form, $form_state) {
  $form['description']['widget'][0]['format']['#allowed_formats'] = ['plain_text'];
}

/**
 * Form submission handler.
 */
function cforge_node_submit($form, $form_state) {
  cforge_node_set_anonaccess($form_state->getFormObject()->getEntity()->id(), $form_state->getValue('publicevent'));
}

/**
 * Implements hook_node_delete().
 */
function cforge_node_delete($node) {
  \Drupal::keyValue('anonaccess')->delete($node->id);
}

/**
 * Determine if a node is intended for anon.
 */
function cforge_node_get_anonaccess($nid) {
  return \Drupal::keyValue('anonaccess')->get($nid);
}

/**
 * Form submit callback.
 *
 * Set the anonaccess flag for a node.
 */
function cforge_node_set_anonaccess($nid, $value) {
  \Drupal::keyValue('anonaccess')
    ->set($nid, $value);
}

/**
 * Determine the flavour of the current site.
 */
function cforge_get_flavour() {
  foreach (CFORGE_FLAVOURS as $mod => $name) {
    if (\Drupal::moduleHandler()->moduleExists($mod)) {
      return $mod;
    }
  }
  throw new \Exception('no flavour installed!');
}

/**
 * Make a list of of all the neighbourhood entities.
 *
 * @return Neighbourhood[]
 *   Both keys and values are the entity label, order alphabetically.
 *
 * @todo can we use Mcapi::entityLabelList($entity_type_id);
 */
function cforge_list_neighbourhoods() {
  $options = [];
  $nids = \Drupal::entityQuery('node')
    ->condition('type', 'neighbourhood')
    ->execute();
  foreach (Node::LoadMultiple($nids) as $nid => $node) {
    $options[] = $node->label();
  }
  sort($options);
  return array_combine($options, $options);
}

/**
 * Check if the passed user lives in the same neighbourhood as the current user.
 *
 * @param \Drupal\user\UserInterface $user
 *   Fully loaded user entity.
 *
 * @result boolean
 *   True if the passed user is in the same neighbourhood as the currentUser.
 */
function cforge_same_neighbourhood(UserInterface $user) {
  // There has to be a more elegant way of addressing subfields.
  $currentuser = User::load(\Drupal::currentuser()->id());
  return $currentuser->address->getValue()[0]['dependent_locality']
    === $user->address->getValue()[0]['dependent_locality'];
}

/**
 * Determine whether the referral system is active according to cforge settings.
 *
 * @return bool
 *   TRUE if the referral fee is not 0 and there is a source wallet.
 */
function cforge_referral() {
  $conf = \Drupal::Config('cforge.settings');
  return !empty_worths_config($conf->get('referral_fee')) && $conf->get('referral_src');
}


/**
 * Implements hook_ENTITY_TYPE_insert().
 *
 * Modify default content view to remove unwanted bulk actions. Unfortunately I
 * can't see how to do this while installing. since the view hasn't been created
 * by the time of cforge_install
 */
function cforge_view_insert($view) {
  if ($view->id() == 'content') {
    $display = &$view->getDisplay('default');
    $display['display_options']['fields']['node_bulk_form']['include_exclude'] = 'include';
    $display['display_options']['fields']['node_bulk_form']['selected_actions'] = [
      'node_delete_action',
      'node_publish_action',
      'node_unpublish_action',
    ];
    $view->save();
  }

}
