<?php

namespace Drupal\cforge;
use \Drupal\node\Entity\Node;
use Drupal\taxonomy\Entity\Term;
use Drupal\node\Entity\NodeType;

/**
 * Base class to create demo data in any lanaguage.
 */
abstract class DemoUserData {

  public static $firstNames = [];
  public static $lastNames = [];
  public static $names = [];
  public static $streets = [];
  public static $hoods = [];

  /**
   * Get a random neighbourhood.
   */
  public static function randNeighbourhood() {
    static $hoods = [];
    if (empty($hoods)) {
      $nodes = \Drupal::entityTypeManager()
        ->getStorage('node')
        ->loadByProperties(['type' => 'neighbourhood']);
      foreach ($nodes as $node) {
        $hoods[] = $node->getTitle();
      }
    }
    if (empty($hoods)) {
      drupal_set_message('no neighbourhoods created yet');
    }
    return $hoods[array_rand($hoods)];
  }

  /**
   * Get a random street.
   */
  public static function randStreet() {
    $key = array_rand(static::$streets);
    return static::$streets[$key];
  }

  /**
   * Get a random phone number.
   */
  public static function randPhone() {
    return '0' . rand(1000, 9999) . ' ' . rand(100000, 999999);
  }

  /**
   * Create the neighbourhood nodes.
   */
  public static function createNeighbourhoods($hoods = []) {
    if (!$hoods) {
      $hoods = static::$hoods;
    }
    foreach ($hoods as $hood) {
      $exists = \Drupal::entityQuery('node')
        ->condition('type', 'neighbourhood')
        ->condition('title', $hood)
        ->execute();
      if (!$exists) {
        Node::create([
          'type' => 'neighbourhood',
          'title' => trim($hood),
          'uid' => 1,
        ])->save();
      }
    }
  }

  /**
   * Called from cforge_install, the final install task.
   */
  public static function generateDemoDataBatch($install_state) {
    $batch = [];
    if ($install_state['example']) {
      $batch['title'] = t('Generating example content');
      // Create 26 users.
      $flavour = cforge_get_flavour();
      $batch['operations'][] = [
        'generate_content',
        [
          $flavour . '_user',
          [
            'kill' => FALSE,
            'num' => 26,
            'pass' => 'password',
            'roles' => ['trader'],
            'time_range' => strtotime('-2 years'),
          ],
        ],
      ];
      $batch['operations'][] = [
        'generate_content',
        [
          'smallad',
          [
            'kill' => FALSE,
            'num' => 120,
            'type' => 'offer',
            'since' => strtotime('-6 months'),
          ],
        ],
      ];
      $batch['operations'][] = [
        'generate_content',
        [
          'smallad',
          [
            'kill' => FALSE,
            'num' => 40,
            'type' => 'want',
            'since' => strtotime('-6 months'),
          ],
        ],
      ];
      $batch['operations'][] = [
        'generate_content',
        [
          'mcapi_transaction',
          [
            'kill' => FALSE,
            'num' => 200,
            'type' => 'default',
            'time_range' => '-2 years'
          ],
        ],
      ];
      if (\Drupal::moduleHandler()->moduleExists('mcapi_signatures')) {
        $batch['operations'][] = [
          'generate_content',
          [
            'mcapi_transaction',
            [
              'kill' => FALSE,
              'num' => 40,
              'type' => 'default',
              'time_range' => '-2 years'
            ],
          ],
        ];
      }
      $batch['operations'][] = [
        'generate_content',
        [
          'content',
          [
            'kill' => FALSE,
            'num' => 15,
            'node_types' => ['story' => TRUE],
            'max_comments' => 6,
            'title_length' => 8,
          ],
        ],
      ];
      if (NodeType::load('event')) {
        $batch['operations'][] = [
          'generate_content',
          [
            'content',
            [
              'kill' => FALSE,
              'num' => 15,
              'node_types' => ['event' => TRUE],
              'max_comments' => 2,
              'title_length' => 8,
            ],
          ],
        ];
      }
      if (NodeType::load('document')) {
        $batch['operations'][] = [
          'generate_content',
          [
            'content',
            [
              'kill' => FALSE,
              'num' => 15,
              'node_types' => ['document' => TRUE],
              'max_comments' => 0,
              'title_length' => 5,
            ],
          ],
        ];
      }
      if (NodeType::load('image')) {
        // Create another 3 galleries.
        $props = ['vid' => 'galleries', 'format' => 'plain_text'];
        Term::create($props + [
          'name' => t('Annnual street party'),
        ])->save();
        Term::create($props + [
          'name' => t('Gurning competition'),
        ])->save();
        Term::create($props + [
          'name' => t('Harvest'),
        ])->save();
        $batch['operations'][] = [
          'generate_content',
          [
            'content',
            [
              'kill' => FALSE,
              'num' => 50,
              'node_types' => ['image' => TRUE],
              'max_comments' => 4,
              'title_length' => 5,
            ],
          ],
        ];
      }
      $batch['finished'] = ['finish_demo_import'];
      $batch['file'] = 'profiles/cforge/modules/hamlets/cforge.install';
    }
    return $batch;
  }

  /**
   * Return a randomised firstname and last name.
   */
  public static function getName() {
    static $firstnames, $lastnames = [];
    if (!$firstnames) {
      $firstnames = static::$firstNames;
      shuffle($firstnames);
    }
    if (!$lastnames) {
      $lastnames = static::$lastNames;
      shuffle($lastnames);
    }
    return [array_pop($firstnames), array_pop($lastnames)];
  }

  /**
   * Return the number of demo user names.
   */
  public static function countUsers() {
    return count(static::$firstNames);
  }

}
